% Demo utilizing a fast multigrid solver for geometric active contour
% models, showing hot to use the corresponding gacs() function
% 
% For a detailed description of the method see the paper
% G. Papandreou and P. Maragos,
% Multigrid Geometric Active Contour Models,
% IEEE Transactions on Image Processing, vol. 16, no. 1, pp. 229-240, Jan. 2007.
% (available from http://cvsp.cs.ntua.gr/publications.shtm)
%
% George Papandreou, gpapan@cs.ntua.gr
% 23-Dec-2006

% add the path to gacs.mexglx/gacs.dll
%addpath('../MEX');

% print help message
%gacs;

% read image
I = im2double(imread('3shapes.pgm'));
max_I = max(I(:));
I = I / max_I;
[m,n,c] = size(I);

% Initial position of the front
init_type=1;
switch init_type
 case 1 % rect hole in the middle
  r1=0.1; r2=0.9;
  mask = logical(zeros([m n])); 
  mask(round(r1*m):round(r2*m), round(r1*n):round(r2*n)) = true;
 otherwise
  error('wrong example');
end

% edge indicator function
hsize=5; sigma=2;
h = fspecial('gaussian',hsize,sigma);
Is = imfilter(I,h,'symmetric');
K_edge = 0.01;
[Ix Iy] = gradient(Is);
grad_mag = Ix.^2+Iy.^2;
g = 1./(1+(grad_mag/K_edge).^2);

% chan-vese region-based additional rhs terms
alpha=1;
c_in=0;  % mean intensity in the foreground
c_out=1;  % mean intensity in the background
arhs = alpha/2*((I-c_in).^2-(I-c_out).^2);

% other parameters to the solvers
params.problem_conv_dist = 5;
params.problem_balloon = 0.0;
params.evolver_dt = 100;
params.evolver_steps = 15;

tic
[u, num_iters] = gacs(mask, g, arhs, params);
toc
disp(sprintf('Convergence after %d iterations.', num_iters));

% find zero level set
ls_mask = abs(u)<2;

% mark image on the level set
ls_color=[1 0 0];
I_res = zeros([m n 3]);
for i=1:3,
  I_ch = I;
  I_ch(ls_mask) = ls_color(i);
  I_res(:,:,i) = I_ch;
end

% show results
figure(gcf);
subplot(2,2,1), imshow(I,[]); title('original image');
subplot(2,2,2), imshow(mask,[]); title('initial segmentation');
subplot(2,2,3), imshow(I_res,[]); title('final segmentation');
subplot(2,2,4), [C1,h]=contour(u); clabel(C1,h); axis ij, title('distance transform');